@@ -7,12 +7,14 @@ import re
 import warnings
 from fractions import Fraction
 from functools import partial
+from inspect import signature
 from typing import Any, Callable, Dict, List, NoReturn, Optional, Set, Union
 
 import jsonschema
 from hypothesis import assume, provisional as prov, strategies as st
 from hypothesis.errors import HypothesisWarning, InvalidArgument
 from hypothesis.internal.conjecture import utils as cu
+from hypothesis.strategies._internal.regex import regex_strategy
 
 from ._canonicalise import (
     FALSEY,
@@ -41,6 +43,17 @@ JSON_STRATEGY: st.SearchStrategy[JSONType] = st.recursive(
 )
 _FORMATS_TOKEN = object()
 
+from_js_regex: Callable[[str], st.SearchStrategy[str]] = st.from_regex
+if len(signature(regex_strategy).parameters) == 3:  # pragma: no branch
+    # On Hypothesis >= 6.31.6, we can use this to get the ECMA semantics of "$".
+    # Conditionally-defined so that we degrade relatively gracefully if you update
+    # Hypothesis but not hypothesis-jsonschema once we have a more general fix.
+
+    def from_js_regex(pattern: str) -> st.SearchStrategy[str]:
+        return regex_strategy(
+            pattern, fullmatch=False, _temp_jsonschema_hack_no_end_newline=True
+        )
+
 
 def merged_as_strategies(
     schemas: List[Schema], custom_formats: Optional[Dict[str, st.SearchStrategy[str]]]
@@ -59,10 +72,10 @@ def merged_as_strategies(
             continue
         s = merged([inputs[g] for g in group])
         if s is not None and s != FALSEY:
-            validators = [make_validator(s) for s in schemas]
+            validators = [make_validator(s).is_valid for s in schemas]
             strats.append(
                 from_schema(s, custom_formats=custom_formats).filter(
-                    lambda obj: all(v.is_valid(obj) for v in validators)
+                    lambda obj: all(v(obj) for v in validators)
                 )
             )
             combined.update(group)
@@ -155,13 +168,13 @@ def __from_schema(
         return st.nothing()
     if schema == TRUTHY:
         return JSON_STRATEGY
+    assert isinstance(schema, dict)
+    if schema.get("$schema") == "http://json-schema.org/draft-03/schema#":
+        raise InvalidArgument("Draft-03 schemas are not supported")
     # Only check if declared, lest we error on inner non-latest-draft schemata.
     if "$schema" in schema:
         jsonschema.validators.validator_for(schema).check_schema(schema)
-        if schema["$schema"] == "http://json-schema.org/draft-03/schema#":
-            raise InvalidArgument("Draft-03 schemas are not supported")
 
-    assert isinstance(schema, dict)
     # Now we handle as many validation keywords as we can...
     # Applying subschemata with boolean logic
     if "not" in schema:
@@ -257,7 +270,7 @@ def number_schema(schema: dict) -> st.SearchStrategy[float]:
         exclude_min=exclude_min,
         exclude_max=exclude_max,
         # Filter out negative-zero as it does not exist in JSON
-    ).filter(lambda n: n != 0 or math.copysign(1, n) == 1)
+    ).map(lambda n: n if n != 0 else abs(n))
 
 
 def rfc3339(name: str) -> st.SearchStrategy[str]:
@@ -291,9 +304,9 @@ def rfc3339(name: str) -> st.SearchStrategy[str]:
         return st.one_of(st.just("Z"), rfc3339("time-numoffset"))
     if name == "partial-time":
         return st.times().map(str)
-    if name == "date" or name == "full-date":
+    if name in ("date", "full-date"):
         return st.dates().map(str)
-    if name == "time" or name == "full-time":
+    if name in ("time", "full-time"):
         return st.tuples(rfc3339("partial-time"), rfc3339("time-offset")).map("".join)
     assert name == "date-time"
     return st.tuples(rfc3339("full-date"), rfc3339("full-time")).map("T".join)
@@ -429,7 +442,7 @@ def string_schema(
     elif "pattern" in schema:
         try:
             re.compile(schema["pattern"])
-            strategy = st.from_regex(schema["pattern"])
+            strategy = from_js_regex(schema["pattern"])
         except re.error as err:
             # Patterns that are invalid in Python, or just malformed
             _warn_invalid_regex(schema["pattern"], err)
@@ -575,18 +588,19 @@ def object_schema(
     dep_schemas = {k: v for k, v in dependencies.items() if k not in dep_names}
     del dependencies
 
+    valid_name = make_validator(names).is_valid
+    known_optional_names: List[str] = sorted(
+        set(filter(valid_name, set(dep_names).union(dep_schemas, properties)))
+        - set(required)
+    )
     name_strats = (
-        st.sampled_from(sorted(dep_names) + sorted(dep_schemas) + sorted(properties))
-        if (dep_names or dep_schemas or properties)
-        else st.nothing(),
         from_schema(names, custom_formats=custom_formats)
         if additional_allowed
         else st.nothing(),
-        st.one_of([st.from_regex(p) for p in sorted(patterns)]),
-    )
-    all_names_strategy = st.one_of([s for s in name_strats if not s.is_empty]).filter(
-        make_validator(names).is_valid
+        st.sampled_from(known_optional_names) if known_optional_names else st.nothing(),
+        st.one_of([from_js_regex(p).filter(valid_name) for p in sorted(patterns)]),
     )
+    all_names_strategy = st.one_of([s for s in name_strats if not s.is_empty])
 
     @st.composite  # type: ignore
     def from_object_schema(draw: Any) -> Any:
